# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/finsets',
                'doc_host': 'https://ionmihai.github.io',
                'git_url': 'https://github.com/ionmihai/finsets',
                'lib_path': 'finsets'},
  'syms': { 'finsets.cli': {'finsets.cli.search': ('cli.html#search', 'finsets/cli.py')},
            'finsets.core': { 'finsets.core.all_metadata': ('core.html#all_metadata', 'finsets/core.py'),
                              'finsets.core.features_metadata': ('core.html#features_metadata', 'finsets/core.py'),
                              'finsets.core.raw_metadata': ('core.html#raw_metadata', 'finsets/core.py'),
                              'finsets.core.search': ('core.html#search', 'finsets/core.py')},
            'finsets.fred.fred': { 'finsets.fred.fred.clean': ('00_fred/fred.html#clean', 'finsets/fred/fred.py'),
                                   'finsets.fred.fred.common_raw_vars': ('00_fred/fred.html#common_raw_vars', 'finsets/fred/fred.py'),
                                   'finsets.fred.fred.download': ('00_fred/fred.html#download', 'finsets/fred/fred.py'),
                                   'finsets.fred.fred.get_series_info': ('00_fred/fred.html#get_series_info', 'finsets/fred/fred.py'),
                                   'finsets.fred.fred.search': ('00_fred/fred.html#search', 'finsets/fred/fred.py')},
            'finsets.fred.fred_api': { 'finsets.fred.fred_api.Fred': ('00_fred/fred_api.html#fred', 'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.__do_series_search': ( '00_fred/fred_api.html#fred.__do_series_search',
                                                                                          'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.__fetch_data': ( '00_fred/fred_api.html#fred.__fetch_data',
                                                                                    'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.__get_search_results': ( '00_fred/fred_api.html#fred.__get_search_results',
                                                                                            'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.__init__': ( '00_fred/fred_api.html#fred.__init__',
                                                                                'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred._parse': ( '00_fred/fred_api.html#fred._parse',
                                                                              'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series': ( '00_fred/fred_api.html#fred.get_series',
                                                                                  'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_all_releases': ( '00_fred/fred_api.html#fred.get_series_all_releases',
                                                                                               'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_as_of_date': ( '00_fred/fred_api.html#fred.get_series_as_of_date',
                                                                                             'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_first_release': ( '00_fred/fred_api.html#fred.get_series_first_release',
                                                                                                'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_info': ( '00_fred/fred_api.html#fred.get_series_info',
                                                                                       'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_latest_release': ( '00_fred/fred_api.html#fred.get_series_latest_release',
                                                                                                 'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.get_series_vintage_dates': ( '00_fred/fred_api.html#fred.get_series_vintage_dates',
                                                                                                'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.search': ( '00_fred/fred_api.html#fred.search',
                                                                              'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.search_by_category': ( '00_fred/fred_api.html#fred.search_by_category',
                                                                                          'finsets/fred/fred_api.py'),
                                       'finsets.fred.fred_api.Fred.search_by_release': ( '00_fred/fred_api.html#fred.search_by_release',
                                                                                         'finsets/fred/fred_api.py')},
            'finsets.papers.hassan_etal_2019': { 'finsets.papers.hassan_etal_2019.clean': ( '02_papers/hassan_etal_2019.html#clean',
                                                                                            'finsets/papers/hassan_etal_2019.py'),
                                                 'finsets.papers.hassan_etal_2019.download': ( '02_papers/hassan_etal_2019.html#download',
                                                                                               'finsets/papers/hassan_etal_2019.py'),
                                                 'finsets.papers.hassan_etal_2019.source_url': ( '02_papers/hassan_etal_2019.html#source_url',
                                                                                                 'finsets/papers/hassan_etal_2019.py'),
                                                 'finsets.papers.hassan_etal_2019.variables': ( '02_papers/hassan_etal_2019.html#variables',
                                                                                                'finsets/papers/hassan_etal_2019.py')},
            'finsets.wrds.compa': { 'finsets.wrds.compa.book_equity': ('01_wrds/compa.html#book_equity', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.cashflow_vars': ('01_wrds/compa.html#cashflow_vars', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.clean': ('01_wrds/compa.html#clean', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.default_raw_vars': ('01_wrds/compa.html#default_raw_vars', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.download': ('01_wrds/compa.html#download', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.investment_vars': ('01_wrds/compa.html#investment_vars', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.issuance_vars': ('01_wrds/compa.html#issuance_vars', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.leverage_vars': ('01_wrds/compa.html#leverage_vars', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.liquidity_vars': ('01_wrds/compa.html#liquidity_vars', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.payout_vars': ('01_wrds/compa.html#payout_vars', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.profitability_vars': ( '01_wrds/compa.html#profitability_vars',
                                                                               'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.raw_metadata': ('01_wrds/compa.html#raw_metadata', 'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.raw_metadata_extra': ( '01_wrds/compa.html#raw_metadata_extra',
                                                                               'finsets/wrds/compa.py'),
                                    'finsets.wrds.compa.tobin_q': ('01_wrds/compa.html#tobin_q', 'finsets/wrds/compa.py')},
            'finsets.wrds.crspm': { 'finsets.wrds.crspm.clean': ('01_wrds/crspm.html#clean', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.default_raw_vars': ('01_wrds/crspm.html#default_raw_vars', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.delist_adj_ret': ('01_wrds/crspm.html#delist_adj_ret', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.download': ('01_wrds/crspm.html#download', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.parse_varlist': ('01_wrds/crspm.html#parse_varlist', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.raw_metadata': ('01_wrds/crspm.html#raw_metadata', 'finsets/wrds/crspm.py'),
                                    'finsets.wrds.crspm.raw_metadata_extra': ( '01_wrds/crspm.html#raw_metadata_extra',
                                                                               'finsets/wrds/crspm.py')},
            'finsets.wrds.linking': { 'finsets.wrds.linking.gvkey_permno_a': ( '01_wrds/linking.html#gvkey_permno_a',
                                                                               'finsets/wrds/linking.py'),
                                      'finsets.wrds.linking.gvkey_permno_m': ( '01_wrds/linking.html#gvkey_permno_m',
                                                                               'finsets/wrds/linking.py'),
                                      'finsets.wrds.linking.gvkey_permno_q': ( '01_wrds/linking.html#gvkey_permno_q',
                                                                               'finsets/wrds/linking.py')},
            'finsets.wrds.wrds_api': { 'finsets.wrds.wrds_api.Connection': ('01_wrds/wrds_api.html#connection', 'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__check_schema_perms': ( '01_wrds/wrds_api.html#connection.__check_schema_perms',
                                                                                                  'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__create_pgpass_file_unix': ( '01_wrds/wrds_api.html#connection.__create_pgpass_file_unix',
                                                                                                       'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__create_pgpass_file_win32': ( '01_wrds/wrds_api.html#connection.__create_pgpass_file_win32',
                                                                                                        'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__enter__': ( '01_wrds/wrds_api.html#connection.__enter__',
                                                                                       'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__exit__': ( '01_wrds/wrds_api.html#connection.__exit__',
                                                                                      'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__get_schema_for_view': ( '01_wrds/wrds_api.html#connection.__get_schema_for_view',
                                                                                                   'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__get_user_credentials': ( '01_wrds/wrds_api.html#connection.__get_user_credentials',
                                                                                                    'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__init__': ( '01_wrds/wrds_api.html#connection.__init__',
                                                                                      'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__make_sa_engine_conn': ( '01_wrds/wrds_api.html#connection.__make_sa_engine_conn',
                                                                                                   'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.__write_pgpass_file': ( '01_wrds/wrds_api.html#connection.__write_pgpass_file',
                                                                                                 'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.close': ( '01_wrds/wrds_api.html#connection.close',
                                                                                   'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.connect': ( '01_wrds/wrds_api.html#connection.connect',
                                                                                     'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.create_pgpass_file': ( '01_wrds/wrds_api.html#connection.create_pgpass_file',
                                                                                                'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.describe_table': ( '01_wrds/wrds_api.html#connection.describe_table',
                                                                                            'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.get_row_count': ( '01_wrds/wrds_api.html#connection.get_row_count',
                                                                                           'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.get_table': ( '01_wrds/wrds_api.html#connection.get_table',
                                                                                       'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.list_libraries': ( '01_wrds/wrds_api.html#connection.list_libraries',
                                                                                            'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.list_tables': ( '01_wrds/wrds_api.html#connection.list_tables',
                                                                                         'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.load_library_list': ( '01_wrds/wrds_api.html#connection.load_library_list',
                                                                                               'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.Connection.raw_sql': ( '01_wrds/wrds_api.html#connection.raw_sql',
                                                                                     'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.NotSubscribedError': ( '01_wrds/wrds_api.html#notsubscribederror',
                                                                                     'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.SchemaNotFoundError': ( '01_wrds/wrds_api.html#schemanotfounderror',
                                                                                      'finsets/wrds/wrds_api.py'),
                                       'finsets.wrds.wrds_api.download': ('01_wrds/wrds_api.html#download', 'finsets/wrds/wrds_api.py')}}}
